/*******************************************************************************
*
*	TITLE:		beacon.c 
*
*	VERSION:	1.0                           
*
*	DATE:		09-Jan-2004
*
*	AUTHOR:		R. Kevin Watson
*
*	COMMENTS:	This file contains the 2004 FRC infrared beacon code.
*				This code is designed to run on the EDU controller.
*				Numbers within brackets refers to the PIC18F8520 data sheet
*				page number where additional information can be found.
*
*				This code assumes that the "type-0" beacon driver and LED are
*				driven by PWM 1 and the "type-1" beacon driver and LED are
*				driven by PWM 2.
*
********************************************************************************
*
*	Change log:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	28-Nov-2003  0.1  RKW Original
*	09-Jan-2004  1.0  RKW - This version used for kick-off demo
*
*******************************************************************************/

#include "ifi_picdefs.h"
#include "beacon.h"

/*******************************************************************************
*
*	FUNCTION:		Initialize_Beacon()
*
*	PURPOSE:		Initializes the infrared beacon code.
*
*	CALLED FROM:	User_Initialization()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:
*
*******************************************************************************/
void Initialize_Beacon(void)  
{
	// initialize PWM 1, which generates type 0 beacon waveforms
	TRISCbits.TRISC1 = 0;		// make sure CCP2/PWM1 is setup as an output [110]
	CCPR2L = PWM_HIGH_PERIOD;	// set CCP2/PWM 1 duty cycle (defined in beacon.h) [

	// initialize PWM 2, which generates type 1 beacon waveforms
	TRISGbits.TRISG0 = 0;		// make sure CCP3/PWM2 is setup as an output [121]
	CCPR3L = PWM_HIGH_PERIOD;	// set CCP3/PWM 2 duty cycle (defined in beacon.h)
		
	// initialize timer 2, which generates the 40KHz IR carrier frequency
	PR2 = PWM_TOTAL_PERIOD;		// set PWM frequency (defined in beacon.h) [154]
	TMR2 = 0;					// clear timer2 register [142]	
	T2CONbits.T2OUTPS0 = 0;		// setup 1:1 postscale [141]
	T2CONbits.T2OUTPS1 = 0;
	T2CONbits.T2OUTPS2 = 0;
	T2CONbits.T2OUTPS3 = 0;
	T2CONbits.T2CKPS0 = 0;		// setup 1:1 prescale [141]
	T2CONbits.T2CKPS1 = 0;
	T2CONbits.TMR2ON = 1;		// start timer 2 [141]

	// initialize timer 0, which provides timing control for the beacon 
	//waveform generation state machine
	INTCON2bits.TMR0IP = 0;		// timer 0 will use the low priority interrupt [90]
	INTCONbits.TMR0IE = 1;		// enable timer 0 roll-over interrupt [89]
	T0CONbits.T0PS0 = 0;		// set timer 0 prescaler to 8:1 [131]
	T0CONbits.T0PS1 = 1;
	T0CONbits.T0PS2 = 0;
	T0CONbits.PSA = 0;			// timer 0 will use the prescaler [131]
	T0CONbits.T0CS = 0;			// timer 0 uses the 10MHz instruction cycle clock [131]
	T0CONbits.T08BIT = 0;		// timer 0 is 16 bits wide [131]
	T0CONbits.TMR0ON = 1;		// start timer 0 [131]
}

/*******************************************************************************
*
*	FUNCTION:		Timer_0_Int_Handler()
*
*	PURPOSE:		This code is called in response to a timer 0 
*					roll-over interrupt.
*
*	CALLED FROM:	User_Routines_Fast()
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
*
*	COMMENTS:		Timer 0 is used to control the beacon state
*					machine timing.
*
*******************************************************************************/
void Timer_0_Int_Handler(void)
{
static unsigned char BeaconState = BEACON_0_ON;

	switch(BeaconState)
	{
		case BEACON_0_ON:
	    BeaconState = BETWEEN_BEACON_0_AND_1;// update state variable for next state
		TMR0H = HIBYTE(BEACON_OFF_PERIOD);	// setup timer 0 delay
    	TMR0L = LOBYTE(BEACON_OFF_PERIOD);
		CCP2CON = PWM_OFF;					// turn off the CCP2/PWM1 output
		break;

		case BETWEEN_BEACON_0_AND_1:
    	BeaconState = BEACON_1_ON;			// update state variable for next state
		TMR0H = HIBYTE(BEACON_1_ON_PERIOD);	// setup timer 0 delay
    	TMR0L = LOBYTE(BEACON_1_ON_PERIOD);
    	CCP3CON = PWM_ON;					// turn on the CCP3/PWM2 output
		break;

		case BEACON_1_ON:
	    BeaconState = BETWEEN_BEACON_1_AND_0;// update state variable for next state
		TMR0H = HIBYTE(BEACON_OFF_PERIOD);	// setup timer 0 delay
    	TMR0L = LOBYTE(BEACON_OFF_PERIOD);
    	CCP3CON = PWM_OFF;					// turn off the CCP3/PWM2 output
		break;

		case BETWEEN_BEACON_1_AND_0:		
    	BeaconState = BEACON_0_ON;			// update state variable for next state
		TMR0H = HIBYTE(BEACON_0_ON_PERIOD);	// setup timer 0 delay
    	TMR0L = LOBYTE(BEACON_0_ON_PERIOD);
    	CCP2CON = PWM_ON;					// turn on the CCP2/PWM1 output
		break;
	}
}
